<?php
/**
 * Displays the set-up phase.
 *
 * @package mod_wordcards
 * @author  Frédéric Massart - FMCorz.net
 */

require_once(__DIR__ . '/../../config.php');

use \mod_wordcards\utils;
use \mod_wordcards\constants;

$cmid = required_param('id', PARAM_INT);
$termid = optional_param('termid', null, PARAM_INT);
$action = optional_param('action', null, PARAM_ALPHA);

$mod = mod_wordcards_module::get_by_cmid($cmid);
$course = $mod->get_course();
$cm = $mod->get_cm();
$modulecontext = context_module::instance($cm->id);

require_login($course, true, $cm);
$mod->require_manage();

$modid = $mod->get_id();
$pagetitle = get_string('setup', 'mod_wordcards');
$baseurl = new moodle_url('/mod/wordcards/setup.php', ['id' => $cmid]);
$formurl = new moodle_url($baseurl);
$term = null;

$PAGE->set_url($baseurl);
$PAGE->navbar->add($pagetitle, $PAGE->url);
$PAGE->set_heading(format_string($course->fullname, true, [context_course::instance($course->id)]));
$PAGE->set_title($pagetitle);

$output = $PAGE->get_renderer('mod_wordcards');

if ($action == 'delete') {
    confirm_sesskey();
    $mod->delete_term($termid);
    // Uncomment when migrating to 3.1.
    // redirect($PAGE->url, get_string('termdeleted', 'mod_wordcards'));
    redirect($PAGE->url);

} else if ($action == 'edit') {
    // Adding those parameters ensures that we confirm that the term belongs to the right module after submission.
    $formurl->param('action', 'edit');
    $formurl->param('termid', 'termid');
    $term = $DB->get_record('wordcards_terms', ['modid' => $modid, 'id' => $termid], '*', MUST_EXIST);
}

$form = new mod_wordcards_form_term($formurl->out(false), ['termid' => $term ? $term->id : 0,'ttslanguage'=>$mod->get_mod()->ttslanguage]);

if (!$term) {
    $term = new stdClass();
    $term->id=null;
}

//prepare filemanager
$audiooptions= utils::fetch_filemanager_opts('audio');
$imageoptions= utils::fetch_filemanager_opts('image');
file_prepare_standard_filemanager($term, 'audio', $audiooptions, $modulecontext, constants::M_COMPONENT, 'audio', $term->id);
file_prepare_standard_filemanager($term, 'image', $audiooptions, $modulecontext, constants::M_COMPONENT, 'image', $term->id);
//set data to form
$form->set_data($term);

if ($data = $form->get_data()) {

    //if this new add and collect data->id
    $needsupdating = false;
    if (empty($data->termid)) {
        $data->modid = $modid;
        $data->id  = $DB->insert_record('wordcards_terms', $data);
    //else set id to termid
    }else{
        $data->id = $data->termid;
        $needsupdating = true;
    }

    //audio data
    if(!empty( $data->audio_filemanager)){
        $audiooptions = utils::fetch_filemanager_opts('audio');
        //$data->audio_filemanager = $audioitemid;
        $data = file_postupdate_standard_filemanager($data, 'audio', $audiooptions, $modulecontext, constants::M_COMPONENT, 'audio',
                $data->id);
        $needsupdating = true;
    }

    //image data
    if(!empty($data->image_filemanager)){
        $imageoptions = utils::fetch_filemanager_opts('image');
       // $data->image_filemanager = $imageitemid;
        $data = file_postupdate_standard_filemanager($data, 'image', $imageoptions, $modulecontext, constants::M_COMPONENT, 'image',
                $data->id);
        $needsupdating = true;
    }

    if ($needsupdating) {

        $DB->update_record('wordcards_terms', $data);
    }

    //finally redirect
    // Uncomment when migrating to 3.1.
    // redirect($PAGE->url, get_string('termsaved', 'mod_wordcards', $data->term));
    redirect($PAGE->url);

}

echo $output->header();
echo $output->heading($pagetitle);
echo $output->navigation($mod, 'setup');
echo $output->box(get_string('setupinstructions',constants::M_COMPONENT), 'generalbox', 'intro');

$form->display();

$table = new mod_wordcards_table_terms('tblterms', $mod);
$table->define_baseurl($PAGE->url);
$table->out(25, false);

echo $output->footer();
